@@ -37,3 +37,24 @@ test_that("corpus with prob does not work with by", {
         "prob not implemented with by"
     )
 })
+
+test_that("corpus_sample by group works", {
+    corp <- corpus(
+        paste("Document number", seq_len(10)),
+        docvars = data.frame(id = paste0("id", seq_len(10)),
+                             grp = c(rep("A", 5), rep("B", 5)), 
+                             stringsAsFactors = FALSE)
+    )
+    expect_equal(
+        docvars(corpus_sample(corp, size = 2, by = "grp"), "grp"),
+        rep(LETTERS[1:2], each = 2)
+    )
+    expect_equal(
+        docvars(corpus_sample(corp, size = 25, by = "grp", replace = TRUE), "grp"),
+        rep(LETTERS[1:2], each = 25)
+    )
+    expect_equal(
+        docvars(corpus_sample(corp, by = "grp"), "grp"),
+        rep(LETTERS[1:2], each = 5)
+    )
+})
