@@ -2,10 +2,18 @@ context("test spacyr methods")
 
 test_that("test quanteda methods for spacy_parsed objects", {
     load("../data/data_spacy_parsed.rda")
-    expect_equal(docnames(data_spacy_parsed), "text1")
-    expect_equal(ndoc(data_spacy_parsed), 1)
-    expect_equal(ntype(data_spacy_parsed), c(text1 = 7))
-    expect_equal(ntoken(data_spacy_parsed), c(text1 = 7))
+    expect_identical(docnames(data_spacy_parsed), "text1")
+    expect_identical(ndoc(data_spacy_parsed), 1L)
+    expect_identical(ntype(data_spacy_parsed), c(text1 = 7L))
+    expect_identical(ntoken(data_spacy_parsed), c(text1 = 7L))
+})
+
+test_that("test nsentence for spacy_parsed objects", {
+    load("../data/data_spacy_parsed2.rda")
+    expect_identical(
+        nsentence(data_spacy_parsed2), 
+        c(text1 = 2L, text2 = 3L, text3 = 1L)
+    )
 })
 
 test_that("test as.tokens works for spacy_parsed objects", {
